#!/usr/bin/env node
;
var bounded, classModel, classes, domain, figlet, fileExtension, fs, homeDir, legend, loopbackClasses, outputFile, pkg, plantUmlClasses, plantUmlrelations, prg, scanModel, term, title, warning,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

fs = require('fs');

term = require('terminal-kit').terminal;

prg = require('commander');

figlet = require('figlet');

pkg = require("../package");

prg.version(pkg.version).description(pkg.description).option('-i, --input directory <required>', 'directory to scan (./models by default)').option('-o, --output <required>', 'Plantuml filename to generate (./model.txt by default)').option('-d, --domain <required>', 'Domain name of the scanned model (\"myDomain\" by default)').option('-t, --title <required>', 'Gives a title to the model').option('-l, --legend <required>', 'Bottom message').option('-b, --bounded', 'Shows only models of the domain (no outer inheritance shown)');

prg.parse(process.argv);

domain = "myDomain";

plantUmlrelations = "";

plantUmlClasses = "";

classes = [];

loopbackClasses = ["PersistedModel", "Application", "Model", "AccessToken", "ACL", "Change", "Checkpoint", "Email", "KeyValueModel", "Role", "RoleMapping", "Scope", "User"];

fileExtension = ".json";

homeDir = "./models";

outputFile = "./model.txt";

bounded = false;

title = "Domain model";

legend = "generated by " + pkg.name + "@" + pkg.version + " " + (new Date());

if (prg.input != null) {
  homeDir = prg.input;
}

if (prg.output != null) {
  outputFile = prg.output;
}

if (prg.domain != null) {
  domain = prg.domain;
}

if (prg.bounded != null) {
  bounded = prg.bounded;
}

if (prg.title != null) {
  title = prg.title;
}

if (prg.legend != null) {
  legend = prg.legend;
}

scanModel = function() {
  var chunk, cptFiles, e, file, i, len, ref;
  plantUmlClasses = "package " + domain + " #AAAAAA {\n";
  term.green("\nScanning in ").bold(homeDir + "\n");
  cptFiles = 0;
  try {
    ref = fs.readdirSync(homeDir);
    for (i = 0, len = ref.length; i < len; i++) {
      file = ref[i];
      if (file.indexOf(fileExtension) > -1) {
        chunk = classModel(JSON.parse(fs.readFileSync(homeDir + "/" + file, 'utf8')));
        plantUmlClasses += chunk;
        cptFiles++;
      }
    }
  } catch (error) {
    e = error;
    term.red("Something wrong occured: " + e.message + "\n");
    process.exit(0);
  }
  plantUmlClasses += '}\n';
  if (cptFiles === 0) {
    term.red("Warning : no model file found, maybe a wrong path? \n");
  }
  term.green("Done !\n");
  return "@startuml\n title: " + title + "\n caption: " + legend + "\n " + plantUmlClasses + "\n " + plantUmlrelations + "\n @enduml\n";
};

warning = function(className) {
  return term.red("\tWarning : " + className + " belongs to Loopback base model also !\n");
};

classModel = function(model) {
  var attributeList, baseClass, classDefinition, className, key, ref, ref1, ref2, relations, value;
  className = model.name;
  baseClass = model.base;
  term.green("\tClass ").bold("" + className);
  if (baseClass != null) {
    term.green("\u27a1 Class " + baseClass);
  }
  term.green("\n");
  if (indexOf.call(loopbackClasses, className) >= 0) {
    warning(className);
  }
  attributeList = "";
  ref = model.properties;
  for (key in ref) {
    value = ref[key];
    if (value.type != null) {
      attributeList += "\t" + key + " : " + value.type + " \n";
    } else {
      attributeList += "\t" + key + " : " + value + " \n";
    }
  }
  classDefinition = "";

  /*
  Inheritance mgt
   */
  if (baseClass != null) {
    if (indexOf.call(loopbackClasses, baseClass) >= 0) {
      if (!bounded) {
        classDefinition += "loopback." + baseClass + " <|-- " + className + "\n";
      }
    } else {
      classDefinition += baseClass + " <|-- " + className + "\n";
    }
  }

  /*
  Relations mgt
   */
  relations = [];
  ref1 = model.relations;
  for (key in ref1) {
    value = ref1[key];
    term.yellow("\t\trelation '");
    term.yellow.bold(key + "':\n");
    term.yellow("\t\t  model: " + value.model + "\n\t\t  type : " + value.type + "\n\t\t  foreing key: " + value.foreignKey + "\n");
    if (ref2 = value.model, indexOf.call(loopbackClasses, ref2) >= 0) {
      if (!bounded) {
        relations += className + " --> \"" + value.type + "\" loopback." + value.model + ": " + key + "\n";
      }
    } else {
      relations += className + " --> \"" + value.type + "\" " + value.model + ": " + key + "\n";
    }
  }

  /*
  PlantUML class definition
   */
  classDefinition += "class " + className + "{\n " + attributeList + "\n }\n " + relations + "\n";
  return classDefinition;
};

term.yellow(figlet.textSync("" + pkg.name), {
  horizontalLayout: 'full'
});

term.yellow(pkg.version + "\n" + pkg.description + "\n\n");

term.green("Model bounded ? ").bold(bounded + "\n");

term.green("Domain ").bold(domain + "\n");

fs.writeFileSync(outputFile, scanModel());

term.green("Result in ").bold(outputFile + "\n");
