#!/usr/bin/env node
;
var bounded, classModel, classes, descriptionColor, domain, figlet, fileExtension, fs, homeDir, legend, loopbackClasses, no_comment, outputFile, pkg, plantUmlClasses, plantUmlrelations, prg, scanModel, splitDescription, term, title, warning,
  indexOf = [].indexOf || function(item) { for (var i = 0, l = this.length; i < l; i++) { if (i in this && this[i] === item) return i; } return -1; };

fs = require('fs');

term = require('terminal-kit').terminal;

prg = require('commander');

figlet = require('figlet');

pkg = require("../package");

prg.version(pkg.version).description(pkg.description).option('-i, --input directory <required>', 'directory to scan (./models by default)').option('-o, --output <required>', 'Plantuml filename to generate (./model.txt by default)').option('-d, --domain <required>', 'Domain name of the scanned model (\"myDomain\" by default)').option('-t, --title <required>', 'Gives a title to the model').option('-l, --legend <required>', 'Bottom message').option('-b, --bounded', 'Shows only models of the domain (no outer inheritance shown)').option('-n, --no_comment', 'Hides comments linked to attributes');

prg.parse(process.argv);

domain = "myDomain";

plantUmlrelations = "";

plantUmlClasses = "";

classes = [];

loopbackClasses = ["PersistedModel", "Application", "Model", "AccessToken", "ACL", "Change", "Checkpoint", "Email", "KeyValueModel", "Role", "RoleMapping", "Scope", "User"];

fileExtension = ".json";

homeDir = "./models";

outputFile = "./model.txt";

bounded = false;

no_comment = false;

title = "Domain model";

legend = "generated by " + pkg.name + "@" + pkg.version + " " + (new Date());

descriptionColor = "<color:White>";

if (prg.input != null) {
  homeDir = prg.input;
}

if (prg.output != null) {
  outputFile = prg.output;
}

if (prg.domain != null) {
  domain = prg.domain;
}

if (prg.bounded != null) {
  bounded = prg.bounded;
}

if (prg.no_comment != null) {
  no_comment = prg.no_comment;
}

if (prg.title != null) {
  title = prg.title;
}

if (prg.legend != null) {
  legend = prg.legend;
}

scanModel = function() {
  var chunk, cptFiles, e, file, i, len, ref;
  plantUmlClasses = "package " + domain + " #AABBCC{\n";
  term.green("\nScanning in ").bold(homeDir + "\n");
  cptFiles = 0;
  try {
    ref = fs.readdirSync(homeDir);
    for (i = 0, len = ref.length; i < len; i++) {
      file = ref[i];
      if (file.indexOf(fileExtension) > -1) {
        chunk = classModel(JSON.parse(fs.readFileSync(homeDir + "/" + file, 'utf8')));
        plantUmlClasses += chunk;
        cptFiles++;
      }
    }
  } catch (error) {
    e = error;
    term.red("Something wrong occured: " + e.message + "\n");
    process.exit(0);
  }
  plantUmlClasses += '}\n';
  if (cptFiles === 0) {
    term.red("Warning : no model file found, maybe a wrong path? \n");
  }
  term.green("Done !\n");
  return "@startuml\n skinparam roundcorner 10\n skinparam Shadowing true\n skinparam backgroundColor White\n skinparam ArrowColor Black\n skinparam ClassBorderColor Black\n skinparam classFontColor Black\n skinparam classBackgroundColor White\n skinparam ClassHeaderBackgroundColor White\n skinparam classFontSize 14\n skinparam classFontName Aapex\n skinparam defaultFontName Aapex\n skinparam packageFontSize 16\n skinparam packageBackgroundColor Silver\n skinparam titleFontSize 20\n title: " + title + "\n caption: " + legend + "\n " + plantUmlClasses + "\n " + plantUmlrelations + "\n @enduml\n";
};

warning = function(className) {
  return term.red("\tWarning : " + className + " belongs to Loopback base model also !\n");
};

splitDescription = function(mess) {
  var breakers, bump, cpt, cptCarByLine, maxLength, newMess, ref;
  maxLength = 30;
  breakers = [",", "/", ";"];
  if (mess.length <= maxLength) {
    return "" + descriptionColor + mess + "\n";
  }
  newMess = "" + descriptionColor;
  cpt = 0;
  cptCarByLine = 1;
  while (cpt < mess.length) {
    newMess += mess[cpt];
    if (ref = mess[cpt], indexOf.call(breakers, ref) >= 0) {
      newMess += "\n" + descriptionColor;
      cptCarByLine = 1;
    }
    if (cptCarByLine % maxLength === 0) {
      bump = true;
    }
    if (mess[cpt] === " " && bump) {
      newMess += "\n" + descriptionColor;
      cptCarByLine = 1;
      bump = false;
    }
    cpt++;
    cptCarByLine++;
  }
  return newMess + "\n";
};

classModel = function(model) {
  var attributeList, baseClass, classDefinition, className, key, ref, ref1, ref2, relations, value;
  className = model.name;
  baseClass = model.base;
  term.green("\tClass ").bold("" + className);
  if (baseClass != null) {
    term.green(" \u27a1 Class " + baseClass);
  }
  term.green("\n");
  if (indexOf.call(loopbackClasses, className) >= 0 && domain !== "loopback") {
    warning(className);
  }
  attributeList = "";
  ref = model.properties;
  for (key in ref) {
    value = ref[key];
    if (value.type != null) {
      attributeList += "+<b>" + key + "</b> : " + value.type + " \n";
      if ((value.description != null) && !no_comment) {
        attributeList += (splitDescription(value.description)) + "\n";
      }
    } else {
      attributeList += "+<b>" + key + "</b> : " + value + " \n";
    }
  }
  classDefinition = "";

  /*
  Inheritance mgt
   */
  if (baseClass != null) {
    if (indexOf.call(loopbackClasses, baseClass) >= 0) {
      if (!bounded || domain === "loopback") {
        classDefinition += "loopback." + baseClass + " <|-- " + domain + "." + className + "\n";
      }
    } else {
      classDefinition += domain + "." + baseClass + " <|-- " + domain + "." + className + "\n";
    }
  }

  /*
  Relations mgt
   */
  relations = [];
  ref1 = model.relations;
  for (key in ref1) {
    value = ref1[key];
    term.yellow("\t\trelation '");
    term.yellow.bold(key + "':\n");
    term.yellow("\t\t  model: " + value.model + "\n\t\t  type : " + value.type + "\n\t\t  foreing key: " + value.foreignKey + "\n");
    if (ref2 = value.model, indexOf.call(loopbackClasses, ref2) >= 0) {
      if (!bounded || domain === "loopback") {
        relations += domain + "." + className + " --> \"" + value.type + "\" loopback." + value.model + ": " + key + "\n";
      }
    } else {
      relations += domain + "." + className + " --> \"" + value.type + "\" " + value.model + ": " + key + "\n";
    }
  }

  /*
  PlantUML class definition
   */
  classDefinition += "class " + domain + "." + className + " <<(M,DarkOrange)>> #A7C300{\n " + attributeList + "\n }\n " + relations + "\n";
  return classDefinition;
};

term.yellow(figlet.textSync("" + pkg.name), {
  horizontalLayout: 'full'
});

term.yellow(pkg.version + "\n" + pkg.description + "\n\n");

term.green("Model bounded ? ").bold(bounded + "\n");

term.green("Domain ").bold(domain + "\n");

fs.writeFileSync(outputFile, scanModel());

term.green("Result in ").bold(outputFile + "\n");
