# lb2uml.coffee
#
# PlantUML Diag generator from Loopback project definition
#

`#!/usr/bin/env node
`

# 2 previous lines are for mystify coffeescript compiler (see https://coderwall.com/p/k1n70a/compiling-executables-from-coffeescript)


fs = require('fs')
term = require( 'terminal-kit' ).terminal
prg = require('commander')
figlet = require('figlet')
pkg = require("../package")

prg
  .version(pkg.version)
  .description(pkg.description)
  .option('-i, --input directory <required>', 'directory to scan (./models by default)')
  .option('-o, --output <required>', 'Plantuml filename to generate (./model.txt by default)')
  .option('-d, --domain <required>', 'Domain name of the scanned model (\"myDomain\" by default)')
  .option('-t, --title <required>', 'Gives a title to the model')
  .option('-l, --legend <required>', 'Bottom message')
  .option('-b, --bounded', 'Shows only models of the domain (no outer inheritance shown)')
  .option('-n, --no_comment', 'Hides comments linked to attributes')

prg.parse(process.argv)

domain="myDomain"

plantUmlrelations=""
plantUmlClasses=""

classes=[]

# loopback base model

loopbackClasses=[
    "PersistedModel"
    "Application"
    "Model"
    "AccessToken"
    "ACL"
    "Change"
    "Checkpoint"
    "Email"
    "KeyValueModel"
    "Role"
    "RoleMapping"
    "Scope"
    "User"
]

fileExtension=".json"
homeDir="./models"
outputFile="./model.txt"
bounded=false
no_comment=false
title="Domain model"
legend="generated by #{pkg.name}@#{pkg.version} #{new Date()}"
descriptionColor="<color:White>"

homeDir = prg.input if prg.input?  
outputFile = prg.output if prg.output?
domain = prg.domain if prg.domain?
bounded = prg.bounded if prg.bounded?
no_comment = prg.no_comment if prg.no_comment?
title = prg.title if prg.title?
legend = prg.legend if prg.legend?


scanModel= ()->

  plantUmlClasses="package #{domain} #AABBCC{\n"

  term.green("\nScanning in ").bold("#{homeDir}\n")

  cptFiles=0
  try
    for file in fs.readdirSync(homeDir)
      if file.indexOf(fileExtension)>-1  
        chunk=classModel(JSON.parse(fs.readFileSync(homeDir+"/"+file, 'utf8'))) 
        plantUmlClasses+=chunk  
        cptFiles++
  catch e
    term.red("Something wrong occured: #{e.message}\n")  
    process.exit(0)

  plantUmlClasses+='}\n'

  term.red("Warning : no model file found, maybe a wrong path? \n") if cptFiles is 0
  term.green("Done !\n")

  return \
      "
      @startuml\n
      skinparam roundcorner 10\n
      skinparam Shadowing true\n
      skinparam backgroundColor White\n
      skinparam ArrowColor Black\n
      skinparam ClassBorderColor Black\n
      skinparam classFontColor Black\n
      skinparam classBackgroundColor White\n 
      skinparam ClassHeaderBackgroundColor White\n 
      skinparam classFontSize 14\n
      skinparam classFontName Aapex\n
      skinparam defaultFontName Aapex\n
      skinparam packageFontSize 16\n
      skinparam packageBackgroundColor Silver\n
      skinparam titleFontSize 20\n
      title: #{title}\n
      caption: #{legend}\n 
      #{plantUmlClasses}\n
      #{plantUmlrelations}\n
      @enduml\n
      "  

warning= (className) ->      
  term.red("\tWarning : #{className} belongs to Loopback base model also !\n")

splitDescription= (mess)->
  
  maxLength = 30
  breakers=[",","/",";"]

  if mess.length<= maxLength then return "#{descriptionColor}#{mess}\n"

  newMess="#{descriptionColor}"
  cpt=0
  cptCarByLine=1

  while cpt<mess.length 
    newMess+=mess[cpt]
    if mess[cpt] in breakers  
      newMess+="\n#{descriptionColor}" 
      cptCarByLine=1
    bump=true if cptCarByLine%maxLength == 0 
    if mess[cpt] is " " and bump 
      newMess+="\n#{descriptionColor}"
      cptCarByLine=1
      bump=false
    cpt++  
    cptCarByLine++ 
  return "#{newMess}\n"


classModel= (model) ->

  className= model.name
  baseClass= model.base	

  term.green("\tClass ").bold("#{className}")
  term.green(" \u27a1 Class #{baseClass}") if baseClass? 
  term.green("\n")
  warning(className) if className in loopbackClasses and domain isnt "loopback"

  attributeList = ""

  for key,value of model.properties
  	if value.type? 
      attributeList+="+<b>#{key}</b> : #{value.type} \n" 
      attributeList+="#{splitDescription(value.description)}\n" if value.description? and !no_comment
    else
      attributeList+="+<b>#{key}</b> : #{value} \n" 
  classDefinition=""

  ###
  Inheritance mgt
  ###  
  if baseClass? 
    if baseClass in loopbackClasses 
      classDefinition+= "loopback.#{baseClass} <|-- #{domain}.#{className}\n" if not bounded or domain is "loopback"
    else    
      classDefinition+= "#{domain}.#{baseClass} <|-- #{domain}.#{className}\n"  

  ###
  Relations mgt
  ###
  relations=[]

  for key,value of model.relations
    term.yellow("\t\trelation '")
    term.yellow.bold("#{key}':\n")
    term.yellow("\t\t  model: #{value.model}\n\t\t  type : #{value.type}\n\t\t  foreing key: #{value.foreignKey}\n")
    if value.model in loopbackClasses 
      relations+= "#{domain}.#{className} --> \"#{value.type}\" loopback.#{value.model}: #{key}\n" if not bounded or domain is "loopback"
    else
      relations+= "#{domain}.#{className} --> \"#{value.type}\" #{value.model}: #{key}\n"   

  ###
  PlantUML class definition
  ###    
  classDefinition+=
    "
    class #{domain}.#{className} <<(M,DarkOrange)>> #A7C300{\n
    #{attributeList}\n
    }\n
    #{relations}\n
    "
  return classDefinition  

# main

term.yellow(figlet.textSync("#{pkg.name}"),{horizontalLayout: 'full'})
term.yellow("#{pkg.version}\n#{pkg.description}\n\n")
term.green("Model bounded ? ").bold("#{bounded}\n")
term.green("Domain ").bold("#{domain}\n")

fs.writeFileSync(outputFile,scanModel())
term.green("Result in ").bold("#{outputFile}\n")  
